% function [Id,PSF] = apply_degradation(Im,Type,varargin);
%
%  INPUT:
%  Im:    Image to be degraded
%  Type: type of degradation to be applied. Implemented degratations are:
%            'gn' :          gaussian white noise (iid)
%            'blur_gn':  blur plus gaussian white noise
%            'jpeg'        :   jpeq compression
%            'salt_pepper': salt and pepper noise
%  The number of arguments depends on the type of degradation:
%      gn: the variance of the noise.
%      blur_gn: the normalized frequency cut (0-1) of the low-pass filter an the variance of the noise
%      jpeg: the quality of the compression (see imwrite)
%      salt_pepper: noise density (see imnoise)
%
%   OUTPUT:
%   Id: degraded image
%   PSF: the point spread function of the blurring (this can be just a delta function if there is no blurring).
% 
%    LocalRestorationToolbox
%    See J. Gutierrez, F. Ferri and J. Malo '' IEEE TIP Jan. 2006
% 
 
function [degraded,PSF] = apply_degradation(Im,type,varargin);

if (strcmpi(type,'gn'))

   % This is just to use our function degrade 

   % Cutoff frequency of the blurring operator (in this case no blur)
   fc = 2;
   Val_min = 0.001;

   % Frecuency band of the noise 
   % In this case: white noise
   lim_frec = [0 3];

   % Noise variance
   variance = varargin{1}
   
   [degraded,PSF,H,noise,Blurred]=degrade(Im,fc,Val_min,lim_frec,variance);     
   
elseif  (strcmpi(type,'blur_gn'))

   % This is just to use our function degrade 

   % Cutoff frequency of the blurring operator
   fc = varargin{1};
   Val_min = 0.001;

   % Frecuency band of the noise 
   % In this case: white noise
   lim_frec = [0 3];

   % Noise variance
   variance = varargin{2};

   [degraded,PSF,H,noise,Blurred]=degrade(Im,fc,Val_min,lim_frec,variance);
   
elseif  (strcmpi(type,'jpeg'))

   PSF = zeros(11);
   PSF(6,6)=1;
   
   imwrite(Im/255,'tmp.jpg','Quality',varargin{1});
   degraded = imread('tmp.jpg');
   
elseif  (strcmpi(type,'salt_pepper'))

   PSF = zeros(11);
   PSF(6,6)=1;
   degraded = imnoise(Im/255,'salt & pepper', varargin{1});
   degraded = degraded*255;
else
   error('The requested degradation is not implemented');
end

degraded = double(degraded);
