% function [objeto]=regu_modelo_ar8(Imobservada,b,lambdas,N,M,criterio_lambda,pinta);
%
%   Dada una imagen observada que ha pasado por un proceso de degradacin  representado
%   por PSF y a la que se ha aadido ruido, se intenta regularizar 
%   utilizando Wiener, a partir de una estimacin del espectro 
%   modelizando la seal mediante un modelo autorregresivo 
%   
%      
%      EJEMPLO                                                                        
%                                                                                     
%      [Irest_AR8]=regu_modelo_ar8(BlurredNoisy,PSF,lambdas,N,M,criterio_lambda,pinta);
%      


function [restaurada]=regu_modelo_ar8(Imobservada,b,lambdas,N,M,criterio_lambda,pinta);

[dummy,noiseest]=wiener2(Imobservada,[3 3]);

[fil,col]=size(Imobservada);


nver = fil/N;
nhor = col/N;

objetoparaoverlapping = rand(nver*M,nhor*N);


restaurada=rand([fil,col]); 

a=(M-N)/2;

Im2=ampliaconborde(Imobservada,a);




nlamb=length(lambdas);



% El operador de blurring en el dominio de Fourier

B=freqz2(b,M,M);

B=ifftshift(B);



contadorfilas=0; 

for i=N/2:N:fil
   %fprintf('\nFila %d : ',i)
   contadorfilas = contadorfilas+1;
   contadorcolumnas = 0;

   for j=N/2:N:col
      %fprintf('%d, ',j)
      contadorcolumnas = contadorcolumnas + 1;
 
  
      observado=Im2(i+a-M/2+1:i+a+M/2,j+a-M/2+1:j+a+M/2);
      
                  
      cub=fft2(observado);

      % Ahora construyo un operador para este bloque

      % Obtengo los coeficientes de un modelo AR no-causal
      %
      %          a(8)  a(7)  a(6)
      %          a(5)   x    a(4)
      %          a(3)  a(2)  a(1)
      %
      % donde la x representa el elemento (i,j) que deseamos estimar a
      % partir de los anteriores
      
      [PSD,operador,coef_ar] = ajusta_bloque_ar8(observado);


      % Una vez tenemos el operador restauramos el bloque

      [sol]=restaura_bloque(observado,B,operador,M,N,lambdas,criterio_lambda);


      restaurada(i-N/2+1:i+N/2,j-N/2+1:j+N/2)=sol(a+1:a+N,a+1:a+N);
      
      if (pinta==1)
         imshow(restaurada,[0 256],'notruesize'),pause(0.2);
      end
           
      
      objetoparaoverlapping((contadorfilas-1)*M+1:contadorfilas*M,(contadorcolumnas-1)*M+1:contadorcolumnas*M) = sol;
      

   end
end

restaurada = do_overlap(objetoparaoverlapping,M,N,8);



