% function [rest]=regu_operador(Imobservada,b,lambdas,N,M,F,criterio_lambda,pinta);
%
%   Dada una imagen observada que ha pasado por un proceso de degradacin  representado
%   por K y a la que se ha aadido ruido, se intenta regularizar usando el operador
%   de penalizacin F y parmetros de regularizacin lambdas. N es el tamao del bloque
%   y M es el tamao del bloque ms un borde, esto se hace para que los efectos de borde
%   no se vean en la solucin. 
%   Se devuelve tambien la imagen con los bloques sin procesar para hacer el overlapping
%
%      
%      EJEMPLO                                                                        
%                                                                                     
%      N=16;                                                                          
%      M=44;                                                                          
%                                                                                     
%      Nlambdas=70;                                                                   
%                                                                                     
%      lambdas=logspace(-2,2,Nlambdas);                                               
%                                                                                     
%                                                                                     
%      operador=fft2(filt2dsegder,M,M);                                               
%                                                                                     
%      figure,[Irest]=regu_operador(BlurredNoisy,PSF,lambdas,N,M,operador,criterio_lambda,pinta);
%      


function [restaurada,objetoparaoverlapping]=regu_operador(Imobservada,b,lambdas,N,M,F,criterio_lambda,pinta,varargin);


sinruido=wiener2(Imobservada,[3 3]);

noiseest = mean(var(sinruido-Imobservada));

normaruido = sqrt(noiseest)*M;



[fil,col]=size(Imobservada);

restaurada=rand([fil,col]); 

nver = fil/N;
nhor = col/N;

objetoparaoverlapping = rand(nver*M,nhor*M);


a=(M-N)/2;

Im2=ampliaconborde(Imobservada,a);
 
nlamb=length(lambdas);


%lambdasoptimas=restaurada;

% El operador de blurring en Fourier

B=freqz2(b,M,M);

B=ifftshift(B);



%Im3 = ampliaconborde(Imoriginal,a);


contadorfilas=0; 

%% Estimacin del parametro de regularizacion

%Nbloques = 20;
%
%filas = round(N/2 + rand(Nbloques,1)*(fil-N));
%columnas = round(N/2 + rand(Nbloques,1)*(col-N));
%
%lamb = 0; 
%
%for i=1:Nbloques
%   
%      observado=Im2(filas(i)+a-M/2+1:filas(i)+a+M/2,columnas(i)+a-M/2+1:columnas(i)+a+M/2);
%            
%      [sol,lambda]=restaura_bloque(observado,B,F,M,N,lambdas,2);
%      
%      lamb = lamb + lambda;
%end
%
%lamb = lamb / Nbloques;

%% Fin estimacin del parametro de regularizacion 
 

if (nargin==8)
   
   for i=N/2:N:fil
      %fprintf('\nFila %d : ',i)
      contadorfilas = contadorfilas+1;
      contadorcolumnas = 0;
   
      for j=N/2:N:col
         %fprintf('%d, ',j)
      
         contadorcolumnas = contadorcolumnas + 1;
    
      
         observado=Im2(i+a-M/2+1:i+a+M/2,j+a-M/2+1:j+a+M/2);
         
        % bloqueor=Im3(i+a-M/2+1:i+a+M/2,j+a-M/2+1:j+a+M/2);
              
   
         
         [sol]=restaura_bloque(observado,B,F,M,N,lambdas,criterio_lambda);
         
    
         restaurada(i-N/2+1:i+N/2,j-N/2+1:j+N/2)=sol(a+1:a+N,a+1:a+N);
         
         if (pinta==1)
            imshow(restaurada,[0 256],'notruesize'),pause(0.2);
         end
              
         %lambdasoptimas(i-N/2+1:i+N/2,j-N/2+1:j+N/2)=lcurva.corner;
         
         objetoparaoverlapping((contadorfilas-1)*M+1:contadorfilas*M,(contadorcolumnas-1)*M+1:contadorcolumnas*M) = sol;
          
      end
   end
   
   restaurada = do_overlap(objetoparaoverlapping,M,N,8);
    

else
   centro = varargin{1};
   
   i = centro(1);
   j=centro(2);
   
   observado=Im2(i+a-M/2+1:i+a+M/2,j+a-M/2+1:j+a+M/2);
         
   % bloqueor=Im3(i+a-M/2+1:i+a+M/2,j+a-M/2+1:j+a+M/2);
                          
   [sol]=restaura_bloque(observado,B,F,M,N,lambdas,criterio_lambda);
   
   restaurada = sol;
end
   
   
 
 
 