%   function imagen = do_overlapping(Im,M,N,ov)
%
% A partir de una imagen con bloques de tamao MxM devuelve
% una imagen con bloques de tamao NxN considerando ovelapping
% entre los bloques (para hacer suaves las transiciones)
%
% La forma del overlapping esta definida en la funcion 
% pesos_overlapping
%

function imagen = do_overlapping(Im,M,N,ov)


p=pesos_overlapping(M,N,ov);

[nv nh]=size(Im);

nhor = nh/M;
nver = nv/M; 

imagen = zeros(nhor*N,nver*N);

% Parte central: influencia 9 bloques

for i=2:nver-1
   for j=2:nhor-1
      cf = 1;
      Algo = zeros(2*N+M);
      SumaPesos = zeros(2*N+M);
      for k=i-1:i+1
         cc = 1;
         for r=j-1:j+1
            B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
            B = B.*p;
            
            Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
            SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
            cc = cc+N;
         end
         cf = cf + N;
      end

      Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);
      
      Bloque = Algo(M/2+N/2+1:M/2+N/2+N,M/2+N/2+1:M/2+N/2+N);

      imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
      
   end
end
            

% Parte superior: influencia 6 bloques

i=1;

for j=2:nhor-1
   cf = 1;
   Algo = zeros(M+N,2*N+M);
   SumaPesos = zeros(M+N,2*N+M);
   for k=i:i+1
      cc = 1;
      for r=j-1:j+1
         B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
         B = B.*p;
         
         Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
         SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
         cc = cc+N;
      end
      cf = cf + N;
   end

   Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);
   
   Bloque = Algo(M/2-N/2+1:M/2-N/2+N,M/2+N/2+1:M/2+N/2+N);

   imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
   
end
   
   
% Parte inferior: influencia 6 bloques

i=nver;

for j=2:nhor-1
   cf = 1;
   Algo = zeros(M+N,2*N+M);
   SumaPesos = zeros(M+N,2*N+M);
   for k=i-1:i
      cc = 1;
      for r=j-1:j+1
         B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
         B = B.*p;
         
         Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
         SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
         cc = cc+N;
      end
      cf = cf + N;
   end

   Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);
   
   Bloque = Algo(M/2+N/2+1:M/2+N/2+N,M/2+N/2+1:M/2+N/2+N);

   imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
   
end
   


% Parte izquierda: influencia 6 bloques

j=1;

for i=2:nver-1
   cf = 1;
   Algo = zeros(2*N+M,M+N);
   SumaPesos = zeros(2*N+M,M+N);
   for k=i-1:i+1
      cc = 1;
      for r=j:j+1
         B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
         B = B.*p;
         
         Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
         SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
         cc = cc+N;
      end
      cf = cf + N;
   end

   Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);
   
   Bloque = Algo(M/2+N/2+1:M/2+N/2+N,M/2-N/2+1:M/2-N/2+N);

   imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
   
end
   

% Parte derecha: influencia 6 bloques

j=nhor;

for i=2:nver-1
   cf = 1;
   Algo = zeros(2*N+M,M+N);
   SumaPesos = zeros(2*N+M,M+N);
   for k=i-1:i+1
      cc = 1;
      for r=j-1:j
         B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
         B = B.*p;
         
         Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
         SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
         cc = cc+N;
      end
      cf = cf + N;
   end

   Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);
   
   Bloque = Algo(M/2+N/2+1:M/2+N/2+N,M/2+N/2+1:M/2+N/2+N);

   imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
   
end 


% Esquina superior izquierda: influencia 4 bloques

            
cf = 1;
Algo = zeros(M+N,M+N);
SumaPesos = zeros(M+N,M+N);

i=1;
j=1;

for k=i:i+1
   cc = 1;
   for r=j:j+1
      B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
      B = B.*p;
      
      Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
      SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
      cc = cc+N;
   end
   cf = cf + N;
end

Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);

Bloque = Algo(M/2-N/2+1:M/2-N/2+N,M/2-N/2+1:M/2-N/2+N);

imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
            
            
% Esquina superior derecha: influencia 4 bloques

            
cf = 1;
Algo = zeros(M+N,M+N);
SumaPesos = zeros(M+N,M+N);

i=1;
j=nhor;

for k=i:i+1
   cc = 1;
   for r=j-1:j
      B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
      B = B.*p;
      
      Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
      SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
      cc = cc+N;
   end
   cf = cf + N;
end

Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);

Bloque = Algo(M/2-N/2+1:M/2-N/2+N,M/2+N/2+1:M/2+N/2+N);

imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
                        
            
% Esquina inferior izquierda: influencia 4 bloques

            
cf = 1;
Algo = zeros(M+N,M+N);
SumaPesos = zeros(M+N,M+N);

i=nver;
j=1;

for k=i-1:i
   cc = 1;
   for r=j:j+1
      B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
      B = B.*p;
      
      Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
      SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
      cc = cc+N;
   end
   cf = cf + N;
end

Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);

Bloque = Algo(M/2+N/2+1:M/2+N/2+N,M/2-N/2+1:M/2-N/2+N);

imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
                        
            
% Esquina inferior derecha: influencia 4 bloques

            
cf = 1;
Algo = zeros(M+N,M+N);
SumaPesos = zeros(M+N,M+N);

i=nver;
j=nhor;

for k=i-1:i
   cc = 1;
   for r=j-1:j
      B = Im( (k-1)*M+1:k*M, (r-1)*M+1:r*M);
      B = B.*p;
      
      Algo(cf:cf+M-1,cc:cc+M-1) = Algo(cf:cf+M-1,cc:cc+M-1)+B;
      SumaPesos(cf:cf+M-1,cc:cc+M-1) = SumaPesos(cf:cf+M-1,cc:cc+M-1) +p;
      cc = cc+N;
   end
   cf = cf + N;
end

Algo(SumaPesos~=0) = Algo(SumaPesos~=0)./SumaPesos(SumaPesos~=0);

Bloque = Algo(M/2+N/2+1:M/2+N/2+N,M/2+N/2+1:M/2+N/2+N);

imagen ((i-1)*N+1:i*N,(j-1)*N+1:j*N) = Bloque;
                        
            
                      
         
         