% This demo shows how to denoise an image using different local frequency methods developped by
%  J. Gutierrez, F. J. Ferri and J. Malo 
%  Regularization Operators for Natural Images Based on Nonlinear Perception Models,
%  IEEE Transactions on Image Processing, Vol 15, No 1, pp 189-200, Jan 2006
%
%

clc;
echo off;

if (isempty(who('Id')))
   disp('To run this script there should be an image named Id in the workspace.');
   disp('Id must contain the noise image.');
   disp('apply_degradation.m  can be used to simulate several degradations');
   disp('For instace:');
   disp('    load barbara256');
   disp('    [Id,PSF]=apply_degradation(Im,''gn'',400);');
   return;
end

%% IT IS ASSUMED THAT THE DEGRADED IMAGE IS NAMED Id

% In this case (denoising) the PSF is:

PSF = zeros(11);
PSF(6,6)=1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Nlambdas = 60;

lambdas = logspace(-4,3,Nlambdas);


%% RESTORE IMAGE: SECOND DERIVATIVE OPERATOR

disp(['Restoring image using L2 (' num2str(Nlambdas) ' values of lambda)']);

imrl2=restore(Id,'L2',PSF,lambdas);

figure;
imshow(imrl2/255);
title('Restored (L2)');

disp('Press any key...');
pause;


%% RESTORE IMAGE: CSF OPERATOR

disp(['Restoring image using CSF (' num2str(Nlambdas) ' values of lambda)']);

imrcsf=restore(Id,'CSF',PSF,lambdas);

figure;
imshow(imrcsf/255);
title('Restored (CSF)');

disp('Press any key...');
pause;


%% RESTORE IMAGE: AR4 OPERATOR


disp(['Restoring image using AR4 (' num2str(Nlambdas) ' values of lambda)']);

imrar4=restore(Id,'AR4',PSF,lambdas);

figure;
imshow(imrar4/255);
title('Restored (AR4)');

disp('Press any key...');
pause;



%% RESTORE IMAGE: AR8 OPERATOR


disp(['Restoring image using AR8 (' num2str(Nlambdas) ' values of lambda)']);

imrar8=restore(Id,'AR8',PSF,lambdas);

figure;
imshow(imrar8/255);
title('Restored (AR8)');

disp('Press any key...');
pause;




%% RESTORE IMAGE: AR12 OPERATOR


disp(['Restoring image using AR12 (' num2str(Nlambdas) ' values of lambda)']);

imrar12=restore(Id,'AR12',PSF,lambdas);

figure;
imshow(imrar12/255);
title('Restored (AR12)');

disp('Press any key...');
pause;


%% RESTORE IMAGE: PERCEPTUAL OPERATOR

disp(['Restoring image using PERCEPTUAL (' num2str(Nlambdas) ' values of lambda)']);

imrper=restore(Id,'PER',PSF,lambdas);

figure;
imshow(imrper/255);
title('Restored (PERCEPTUAL)');


