function [IMM]=bayesian_gauss_margin(Im2,Var)

% BAYESIAN_GAUSS_MARGIN bayesian image denoising assuming the image has Gaussian
% marginals in the wavelet domain and is corrupted by Gaussian noise of known variance.
% (see Figueiredo & Nowak 01 for details).
%
% The routine uses a 4 scale QMF orthogonal wavelet domain (see E.Simoncelli
% MatlabPyrTools).
%
% SYNTAX: 
%
%      im_d = bayesian_gauss_margin(im_n,variance);
%
%  Input
%  -----
%    * im_n         = noisy image
%    * variance     = noise variance
%
%  Output
%  ------
%    * im_r         = denoised image 
% 
% REFERENCES:
%
% [Figueiredo01] M. Figueiredo and R. Nowak. Wavelet-based image estimation: 
%                an empirical Bayes approach using Jeffreys noninformative prior. 
%                IEEE Transactions on Image Processing, 10(9):13221331, 2001.
% [Simoncelli97] E. Simoncelli. MatlabPyrTools. Matlab toolbox for wavelet transforms 
%                http://www.cns.nyu.edu/~lcv/software.php

orientaciones=cell(3,1);

orientaciones{1}='H';
orientaciones{2}='V';
orientaciones{3}='D';
pyr=[];
pp=[];
for subb=1:4
    for ori=1:3   
        orientacion=orientaciones{ori};
        
        switch lower(orientacion)
            case 'h'
                sub=1;
            case 'v'
                sub=2;
            case 'd'
                sub=3;
        end
        
        [histograma,valores,subbanda,subbandacrop]=marginal_2(Im2,subb,orientacion,256);
        [x_est]=estimacion_figueiredo(subbanda,Var);
        pyr=[pyr x_est'];
        
    end
end
[QQ,ind]=buildwpyr(Im2,4);
pyr=[pyr QQ(end-255:end)'];
IMM=reconwpyr(pyr',ind);